<?php
declare(strict_types=1);

namespace Inti\Media;

use function realpath;

/**
 * Class ConfigProvider
 *
 * @package Inti\Media
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-media
 */
final class ConfigProvider
{
    /**
     * @return array
     */
    public function __invoke(): array
    {
        return [
            'filesystems' => [
                'image' => [
                    'adapter' => 'image',
                    'options' => [
                        'location' => realpath('public/media'),
                    ],
                ],
            ],

            'dependencies' => [
                'factories' => [
                    'ImageMetadataCache' => Container\ImageMetadataCacheFactory::class,
                    'IntiMedia' => Container\ApplicationFactory::class,
                ],
            ],

            'filesystem_adapters' => [
                'aliases' => [
                    'image' => Filesystem\Adapter\ImageFilesystemAdapter::class,
                ],

                'factories' => [
                    Filesystem\Adapter\ImageFilesystemAdapter::class => Filesystem\Adapter\ImageFilesystemAdapterFactory::class,
                ],
            ],

            'wp' => [
                'plugins' => [
                    Plugin\GathImage::class,
                ],
            ],

            'wp_plugins' => [
                'factories' => [
                    Plugin\GathImage::class => Plugin\GathImageFactory::class,
                ],
            ],
        ];
    }
}
