<?php
declare(strict_types=1);

namespace Inti\Media\Handler;

use Laminas\Diactoros\Response\TextResponse;
use League\Flysystem\FilesystemException;
use League\Flysystem\FilesystemOperator;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\RequestHandlerInterface;
use RuntimeException;
use function md5;

/**
 * Class Upload
 *
 * @package Inti\Media\Handler
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-media
 */
final class Upload implements RequestHandlerInterface
{
    /**
     * @param ServerRequestInterface $request
     * @return ResponseInterface
     * @throws FilesystemException
     */
    public function handle(ServerRequestInterface $request): ResponseInterface
    {
        $this->validateRequest($request);

        /** @var FilesystemOperator $filesystem */
        $filesystem = $request->getAttribute('filesystem');
        $path = $this->getImagePath($request);

        if ($filesystem->fileExists($path)) {
            $responseCode = 200;
        } else {
            $filesystem->write($path, $request->getBody()->getContents());
            $responseCode = 201;
        }

        $image = $filesystem->read($path);
        $md5 = md5($image);

        $path = substr(preg_replace('/webp$/', $md5 . '.webp', $path), 8);
        return new TextResponse($path, $responseCode);
    }

    private function getImagePath(ServerRequestInterface $request): string
    {
        $path = strtolower(rawurldecode($request->getUri()->getPath()));
        $path = preg_replace('/\s+/', '-', $path);
        $path = preg_replace('/[[\]]/', '-', $path);

        preg_match('/(.+)\.(.+)$/', $path, $matches);

        if ($matches[2] !== 'webp') {
            $path = $matches[1] . '.webp';
        }

        return 'image://' . ltrim($path, '/');
    }

    private function validateRequest(ServerRequestInterface $request): void
    {
//        $contentType = $request->getHeaderLine('Content-Type');
//        $allowedTypes = ['image/jpeg', 'image/png', 'image/webp'];

//        if (!in_array($contentType, $allowedTypes)) {
//            throw new RuntimeException(sprintf(
//                '%s: Invalid content type. Can only be one of %s, got %s instead.',
//                __METHOD__,
//                implode(', ', $allowedTypes),
//                $contentType
//            ));
//        }

        $path = $request->getUri()->getPath();

        if (empty($path)) {
            throw new RuntimeException(sprintf(
                '%s: Image path is missing', __METHOD__
            ));
        }
    }
}
